# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _dtv_swig1
else:
    import _dtv_swig1

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)



def high_res_timer_now() -> "gr::high_res_timer_type":
    r"""high_res_timer_now() -> gr::high_res_timer_type"""
    return _dtv_swig1.high_res_timer_now()

def high_res_timer_now_perfmon() -> "gr::high_res_timer_type":
    r"""high_res_timer_now_perfmon() -> gr::high_res_timer_type"""
    return _dtv_swig1.high_res_timer_now_perfmon()

def high_res_timer_tps() -> "gr::high_res_timer_type":
    r"""high_res_timer_tps() -> gr::high_res_timer_type"""
    return _dtv_swig1.high_res_timer_tps()

def high_res_timer_epoch() -> "gr::high_res_timer_type":
    r"""high_res_timer_epoch() -> gr::high_res_timer_type"""
    return _dtv_swig1.high_res_timer_epoch()
STANDARD_DVBS2 = _dtv_swig1.STANDARD_DVBS2

STANDARD_DVBT2 = _dtv_swig1.STANDARD_DVBT2

C1_4 = _dtv_swig1.C1_4

C1_3 = _dtv_swig1.C1_3

C2_5 = _dtv_swig1.C2_5

C1_2 = _dtv_swig1.C1_2

C3_5 = _dtv_swig1.C3_5

C2_3 = _dtv_swig1.C2_3

C3_4 = _dtv_swig1.C3_4

C4_5 = _dtv_swig1.C4_5

C5_6 = _dtv_swig1.C5_6

C7_8 = _dtv_swig1.C7_8

C8_9 = _dtv_swig1.C8_9

C9_10 = _dtv_swig1.C9_10

C13_45 = _dtv_swig1.C13_45

C9_20 = _dtv_swig1.C9_20

C90_180 = _dtv_swig1.C90_180

C96_180 = _dtv_swig1.C96_180

C11_20 = _dtv_swig1.C11_20

C100_180 = _dtv_swig1.C100_180

C104_180 = _dtv_swig1.C104_180

C26_45 = _dtv_swig1.C26_45

C18_30 = _dtv_swig1.C18_30

C28_45 = _dtv_swig1.C28_45

C23_36 = _dtv_swig1.C23_36

C116_180 = _dtv_swig1.C116_180

C20_30 = _dtv_swig1.C20_30

C124_180 = _dtv_swig1.C124_180

C25_36 = _dtv_swig1.C25_36

C128_180 = _dtv_swig1.C128_180

C13_18 = _dtv_swig1.C13_18

C132_180 = _dtv_swig1.C132_180

C22_30 = _dtv_swig1.C22_30

C135_180 = _dtv_swig1.C135_180

C140_180 = _dtv_swig1.C140_180

C7_9 = _dtv_swig1.C7_9

C154_180 = _dtv_swig1.C154_180

C11_45 = _dtv_swig1.C11_45

C4_15 = _dtv_swig1.C4_15

C14_45 = _dtv_swig1.C14_45

C7_15 = _dtv_swig1.C7_15

C8_15 = _dtv_swig1.C8_15

C32_45 = _dtv_swig1.C32_45

C2_9_VLSNR = _dtv_swig1.C2_9_VLSNR

C1_5_MEDIUM = _dtv_swig1.C1_5_MEDIUM

C11_45_MEDIUM = _dtv_swig1.C11_45_MEDIUM

C1_3_MEDIUM = _dtv_swig1.C1_3_MEDIUM

C1_5_VLSNR_SF2 = _dtv_swig1.C1_5_VLSNR_SF2

C11_45_VLSNR_SF2 = _dtv_swig1.C11_45_VLSNR_SF2

C1_5_VLSNR = _dtv_swig1.C1_5_VLSNR

C4_15_VLSNR = _dtv_swig1.C4_15_VLSNR

C1_3_VLSNR = _dtv_swig1.C1_3_VLSNR

C_OTHER = _dtv_swig1.C_OTHER

FECFRAME_SHORT = _dtv_swig1.FECFRAME_SHORT

FECFRAME_NORMAL = _dtv_swig1.FECFRAME_NORMAL

FECFRAME_MEDIUM = _dtv_swig1.FECFRAME_MEDIUM

MOD_QPSK = _dtv_swig1.MOD_QPSK

MOD_16QAM = _dtv_swig1.MOD_16QAM

MOD_64QAM = _dtv_swig1.MOD_64QAM

MOD_256QAM = _dtv_swig1.MOD_256QAM

MOD_8PSK = _dtv_swig1.MOD_8PSK

MOD_8APSK = _dtv_swig1.MOD_8APSK

MOD_16APSK = _dtv_swig1.MOD_16APSK

MOD_8_8APSK = _dtv_swig1.MOD_8_8APSK

MOD_32APSK = _dtv_swig1.MOD_32APSK

MOD_4_12_16APSK = _dtv_swig1.MOD_4_12_16APSK

MOD_4_8_4_16APSK = _dtv_swig1.MOD_4_8_4_16APSK

MOD_64APSK = _dtv_swig1.MOD_64APSK

MOD_8_16_20_20APSK = _dtv_swig1.MOD_8_16_20_20APSK

MOD_4_12_20_28APSK = _dtv_swig1.MOD_4_12_20_28APSK

MOD_128APSK = _dtv_swig1.MOD_128APSK

MOD_256APSK = _dtv_swig1.MOD_256APSK

MOD_BPSK = _dtv_swig1.MOD_BPSK

MOD_BPSK_SF2 = _dtv_swig1.MOD_BPSK_SF2

MOD_8VSB = _dtv_swig1.MOD_8VSB

MOD_OTHER = _dtv_swig1.MOD_OTHER

GI_1_32 = _dtv_swig1.GI_1_32

GI_1_16 = _dtv_swig1.GI_1_16

GI_1_8 = _dtv_swig1.GI_1_8

GI_1_4 = _dtv_swig1.GI_1_4

GI_1_128 = _dtv_swig1.GI_1_128

GI_19_128 = _dtv_swig1.GI_19_128

GI_19_256 = _dtv_swig1.GI_19_256

ROTATION_OFF = _dtv_swig1.ROTATION_OFF

ROTATION_ON = _dtv_swig1.ROTATION_ON

STREAMTYPE_TS = _dtv_swig1.STREAMTYPE_TS

STREAMTYPE_GS = _dtv_swig1.STREAMTYPE_GS

STREAMTYPE_BOTH = _dtv_swig1.STREAMTYPE_BOTH

INPUTMODE_NORMAL = _dtv_swig1.INPUTMODE_NORMAL

INPUTMODE_HIEFF = _dtv_swig1.INPUTMODE_HIEFF

CARRIERS_NORMAL = _dtv_swig1.CARRIERS_NORMAL

CARRIERS_EXTENDED = _dtv_swig1.CARRIERS_EXTENDED

PREAMBLE_T2_SISO = _dtv_swig1.PREAMBLE_T2_SISO

PREAMBLE_T2_MISO = _dtv_swig1.PREAMBLE_T2_MISO

PREAMBLE_NON_T2 = _dtv_swig1.PREAMBLE_NON_T2

PREAMBLE_T2_LITE_SISO = _dtv_swig1.PREAMBLE_T2_LITE_SISO

PREAMBLE_T2_LITE_MISO = _dtv_swig1.PREAMBLE_T2_LITE_MISO

FFTSIZE_2K = _dtv_swig1.FFTSIZE_2K

FFTSIZE_8K = _dtv_swig1.FFTSIZE_8K

FFTSIZE_4K = _dtv_swig1.FFTSIZE_4K

FFTSIZE_1K = _dtv_swig1.FFTSIZE_1K

FFTSIZE_16K = _dtv_swig1.FFTSIZE_16K

FFTSIZE_32K = _dtv_swig1.FFTSIZE_32K

FFTSIZE_8K_T2GI = _dtv_swig1.FFTSIZE_8K_T2GI

FFTSIZE_32K_T2GI = _dtv_swig1.FFTSIZE_32K_T2GI

FFTSIZE_16K_T2GI = _dtv_swig1.FFTSIZE_16K_T2GI

PAPR_OFF = _dtv_swig1.PAPR_OFF

PAPR_ACE = _dtv_swig1.PAPR_ACE

PAPR_TR = _dtv_swig1.PAPR_TR

PAPR_BOTH = _dtv_swig1.PAPR_BOTH

L1_MOD_BPSK = _dtv_swig1.L1_MOD_BPSK

L1_MOD_QPSK = _dtv_swig1.L1_MOD_QPSK

L1_MOD_16QAM = _dtv_swig1.L1_MOD_16QAM

L1_MOD_64QAM = _dtv_swig1.L1_MOD_64QAM

PILOT_PP1 = _dtv_swig1.PILOT_PP1

PILOT_PP2 = _dtv_swig1.PILOT_PP2

PILOT_PP3 = _dtv_swig1.PILOT_PP3

PILOT_PP4 = _dtv_swig1.PILOT_PP4

PILOT_PP5 = _dtv_swig1.PILOT_PP5

PILOT_PP6 = _dtv_swig1.PILOT_PP6

PILOT_PP7 = _dtv_swig1.PILOT_PP7

PILOT_PP8 = _dtv_swig1.PILOT_PP8

VERSION_111 = _dtv_swig1.VERSION_111

VERSION_121 = _dtv_swig1.VERSION_121

VERSION_131 = _dtv_swig1.VERSION_131

RESERVED_OFF = _dtv_swig1.RESERVED_OFF

RESERVED_ON = _dtv_swig1.RESERVED_ON

L1_SCRAMBLED_OFF = _dtv_swig1.L1_SCRAMBLED_OFF

L1_SCRAMBLED_ON = _dtv_swig1.L1_SCRAMBLED_ON

MISO_TX1 = _dtv_swig1.MISO_TX1

MISO_TX2 = _dtv_swig1.MISO_TX2

SHOWLEVELS_OFF = _dtv_swig1.SHOWLEVELS_OFF

SHOWLEVELS_ON = _dtv_swig1.SHOWLEVELS_ON

INBAND_OFF = _dtv_swig1.INBAND_OFF

INBAND_ON = _dtv_swig1.INBAND_ON

EQUALIZATION_OFF = _dtv_swig1.EQUALIZATION_OFF

EQUALIZATION_ON = _dtv_swig1.EQUALIZATION_ON

BANDWIDTH_1_7_MHZ = _dtv_swig1.BANDWIDTH_1_7_MHZ

BANDWIDTH_5_0_MHZ = _dtv_swig1.BANDWIDTH_5_0_MHZ

BANDWIDTH_6_0_MHZ = _dtv_swig1.BANDWIDTH_6_0_MHZ

BANDWIDTH_7_0_MHZ = _dtv_swig1.BANDWIDTH_7_0_MHZ

BANDWIDTH_8_0_MHZ = _dtv_swig1.BANDWIDTH_8_0_MHZ

BANDWIDTH_10_0_MHZ = _dtv_swig1.BANDWIDTH_10_0_MHZ

RO_0_35 = _dtv_swig1.RO_0_35

RO_0_25 = _dtv_swig1.RO_0_25

RO_0_20 = _dtv_swig1.RO_0_20

RO_RESERVED = _dtv_swig1.RO_RESERVED

RO_0_15 = _dtv_swig1.RO_0_15

RO_0_10 = _dtv_swig1.RO_0_10

RO_0_05 = _dtv_swig1.RO_0_05

PILOTS_OFF = _dtv_swig1.PILOTS_OFF

PILOTS_ON = _dtv_swig1.PILOTS_ON

INTERPOLATION_OFF = _dtv_swig1.INTERPOLATION_OFF

INTERPOLATION_ON = _dtv_swig1.INTERPOLATION_ON

class dvb_bbheader_bb(object):
    r"""Proxy of C++ gr::dtv::dvb_bbheader_bb class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", rolloff: "gr::dtv::dvbs2_rolloff_factor_t", mode: "gr::dtv::dvbt2_inputmode_t", inband: "gr::dtv::dvbt2_inband_t", fecblocks: "int", tsrate: "int") -> "gr::dtv::dvb_bbheader_bb::sptr":
        r"""make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvbs2_rolloff_factor_t rolloff, gr::dtv::dvbt2_inputmode_t mode, gr::dtv::dvbt2_inband_t inband, int fecblocks, int tsrate) -> dvb_bbheader_bb_sptr"""
        return _dtv_swig1.dvb_bbheader_bb_make(standard, framesize, rate, rolloff, mode, inband, fecblocks, tsrate)
    __swig_destroy__ = _dtv_swig1.delete_dvb_bbheader_bb

# Register dvb_bbheader_bb in _dtv_swig1:
_dtv_swig1.dvb_bbheader_bb_swigregister(dvb_bbheader_bb)

def dvb_bbheader_bb_make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", rolloff: "gr::dtv::dvbs2_rolloff_factor_t", mode: "gr::dtv::dvbt2_inputmode_t", inband: "gr::dtv::dvbt2_inband_t", fecblocks: "int", tsrate: "int") -> "gr::dtv::dvb_bbheader_bb::sptr":
    r"""dvb_bbheader_bb_make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvbs2_rolloff_factor_t rolloff, gr::dtv::dvbt2_inputmode_t mode, gr::dtv::dvbt2_inband_t inband, int fecblocks, int tsrate) -> dvb_bbheader_bb_sptr"""
    return _dtv_swig1.dvb_bbheader_bb_make(standard, framesize, rate, rolloff, mode, inband, fecblocks, tsrate)

class dvb_bbscrambler_bb(object):
    r"""Proxy of C++ gr::dtv::dvb_bbscrambler_bb class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvb_bbscrambler_bb::sptr":
        r"""make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate) -> dvb_bbscrambler_bb_sptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_make(standard, framesize, rate)
    __swig_destroy__ = _dtv_swig1.delete_dvb_bbscrambler_bb

# Register dvb_bbscrambler_bb in _dtv_swig1:
_dtv_swig1.dvb_bbscrambler_bb_swigregister(dvb_bbscrambler_bb)

def dvb_bbscrambler_bb_make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvb_bbscrambler_bb::sptr":
    r"""dvb_bbscrambler_bb_make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate) -> dvb_bbscrambler_bb_sptr"""
    return _dtv_swig1.dvb_bbscrambler_bb_make(standard, framesize, rate)

class dvb_bch_bb(object):
    r"""Proxy of C++ gr::dtv::dvb_bch_bb class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvb_bch_bb::sptr":
        r"""make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate) -> dvb_bch_bb_sptr"""
        return _dtv_swig1.dvb_bch_bb_make(standard, framesize, rate)
    __swig_destroy__ = _dtv_swig1.delete_dvb_bch_bb

# Register dvb_bch_bb in _dtv_swig1:
_dtv_swig1.dvb_bch_bb_swigregister(dvb_bch_bb)

def dvb_bch_bb_make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvb_bch_bb::sptr":
    r"""dvb_bch_bb_make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate) -> dvb_bch_bb_sptr"""
    return _dtv_swig1.dvb_bch_bb_make(standard, framesize, rate)

class dvb_ldpc_bb(object):
    r"""Proxy of C++ gr::dtv::dvb_ldpc_bb class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvb_ldpc_bb::sptr":
        r"""make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvb_ldpc_bb_sptr"""
        return _dtv_swig1.dvb_ldpc_bb_make(standard, framesize, rate, constellation)
    __swig_destroy__ = _dtv_swig1.delete_dvb_ldpc_bb

# Register dvb_ldpc_bb in _dtv_swig1:
_dtv_swig1.dvb_ldpc_bb_swigregister(dvb_ldpc_bb)

def dvb_ldpc_bb_make(standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvb_ldpc_bb::sptr":
    r"""dvb_ldpc_bb_make(gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvb_ldpc_bb_sptr"""
    return _dtv_swig1.dvb_ldpc_bb_make(standard, framesize, rate, constellation)

class dvbt2_interleaver_bb(object):
    r"""Proxy of C++ gr::dtv::dvbt2_interleaver_bb class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvbt2_interleaver_bb::sptr":
        r"""make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvbt2_interleaver_bb_sptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_make(framesize, rate, constellation)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_interleaver_bb

# Register dvbt2_interleaver_bb in _dtv_swig1:
_dtv_swig1.dvbt2_interleaver_bb_swigregister(dvbt2_interleaver_bb)

def dvbt2_interleaver_bb_make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvbt2_interleaver_bb::sptr":
    r"""dvbt2_interleaver_bb_make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvbt2_interleaver_bb_sptr"""
    return _dtv_swig1.dvbt2_interleaver_bb_make(framesize, rate, constellation)

class dvbt2_modulator_bc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_modulator_bc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(framesize: "gr::dtv::dvb_framesize_t", constellation: "gr::dtv::dvb_constellation_t", rotation: "gr::dtv::dvbt2_rotation_t") -> "gr::dtv::dvbt2_modulator_bc::sptr":
        r"""make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt2_rotation_t rotation) -> dvbt2_modulator_bc_sptr"""
        return _dtv_swig1.dvbt2_modulator_bc_make(framesize, constellation, rotation)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_modulator_bc

# Register dvbt2_modulator_bc in _dtv_swig1:
_dtv_swig1.dvbt2_modulator_bc_swigregister(dvbt2_modulator_bc)

def dvbt2_modulator_bc_make(framesize: "gr::dtv::dvb_framesize_t", constellation: "gr::dtv::dvb_constellation_t", rotation: "gr::dtv::dvbt2_rotation_t") -> "gr::dtv::dvbt2_modulator_bc::sptr":
    r"""dvbt2_modulator_bc_make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt2_rotation_t rotation) -> dvbt2_modulator_bc_sptr"""
    return _dtv_swig1.dvbt2_modulator_bc_make(framesize, constellation, rotation)

class dvbt2_cellinterleaver_cc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_cellinterleaver_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(framesize: "gr::dtv::dvb_framesize_t", constellation: "gr::dtv::dvb_constellation_t", fecblocks: "int", tiblocks: "int") -> "gr::dtv::dvbt2_cellinterleaver_cc::sptr":
        r"""make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_constellation_t constellation, int fecblocks, int tiblocks) -> dvbt2_cellinterleaver_cc_sptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_make(framesize, constellation, fecblocks, tiblocks)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_cellinterleaver_cc

# Register dvbt2_cellinterleaver_cc in _dtv_swig1:
_dtv_swig1.dvbt2_cellinterleaver_cc_swigregister(dvbt2_cellinterleaver_cc)

def dvbt2_cellinterleaver_cc_make(framesize: "gr::dtv::dvb_framesize_t", constellation: "gr::dtv::dvb_constellation_t", fecblocks: "int", tiblocks: "int") -> "gr::dtv::dvbt2_cellinterleaver_cc::sptr":
    r"""dvbt2_cellinterleaver_cc_make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_constellation_t constellation, int fecblocks, int tiblocks) -> dvbt2_cellinterleaver_cc_sptr"""
    return _dtv_swig1.dvbt2_cellinterleaver_cc_make(framesize, constellation, fecblocks, tiblocks)

class dvbt2_framemapper_cc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_framemapper_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", rotation: "gr::dtv::dvbt2_rotation_t", fecblocks: "int", tiblocks: "int", carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", guardinterval: "gr::dtv::dvb_guardinterval_t", l1constellation: "gr::dtv::dvbt2_l1constellation_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", t2frames: "int", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t", inputmode: "gr::dtv::dvbt2_inputmode_t", reservedbiasbits: "gr::dtv::dvbt2_reservedbiasbits_t", l1scrambled: "gr::dtv::dvbt2_l1scrambled_t", inband: "gr::dtv::dvbt2_inband_t") -> "gr::dtv::dvbt2_framemapper_cc::sptr":
        r"""make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt2_rotation_t rotation, int fecblocks, int tiblocks, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvb_guardinterval_t guardinterval, gr::dtv::dvbt2_l1constellation_t l1constellation, gr::dtv::dvbt2_pilotpattern_t pilotpattern, int t2frames, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_inputmode_t inputmode, gr::dtv::dvbt2_reservedbiasbits_t reservedbiasbits, gr::dtv::dvbt2_l1scrambled_t l1scrambled, gr::dtv::dvbt2_inband_t inband) -> dvbt2_framemapper_cc_sptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_make(framesize, rate, constellation, rotation, fecblocks, tiblocks, carriermode, fftsize, guardinterval, l1constellation, pilotpattern, t2frames, numdatasyms, paprmode, version, preamble, inputmode, reservedbiasbits, l1scrambled, inband)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_framemapper_cc

# Register dvbt2_framemapper_cc in _dtv_swig1:
_dtv_swig1.dvbt2_framemapper_cc_swigregister(dvbt2_framemapper_cc)

def dvbt2_framemapper_cc_make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", rotation: "gr::dtv::dvbt2_rotation_t", fecblocks: "int", tiblocks: "int", carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", guardinterval: "gr::dtv::dvb_guardinterval_t", l1constellation: "gr::dtv::dvbt2_l1constellation_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", t2frames: "int", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t", inputmode: "gr::dtv::dvbt2_inputmode_t", reservedbiasbits: "gr::dtv::dvbt2_reservedbiasbits_t", l1scrambled: "gr::dtv::dvbt2_l1scrambled_t", inband: "gr::dtv::dvbt2_inband_t") -> "gr::dtv::dvbt2_framemapper_cc::sptr":
    r"""dvbt2_framemapper_cc_make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt2_rotation_t rotation, int fecblocks, int tiblocks, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvb_guardinterval_t guardinterval, gr::dtv::dvbt2_l1constellation_t l1constellation, gr::dtv::dvbt2_pilotpattern_t pilotpattern, int t2frames, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_inputmode_t inputmode, gr::dtv::dvbt2_reservedbiasbits_t reservedbiasbits, gr::dtv::dvbt2_l1scrambled_t l1scrambled, gr::dtv::dvbt2_inband_t inband) -> dvbt2_framemapper_cc_sptr"""
    return _dtv_swig1.dvbt2_framemapper_cc_make(framesize, rate, constellation, rotation, fecblocks, tiblocks, carriermode, fftsize, guardinterval, l1constellation, pilotpattern, t2frames, numdatasyms, paprmode, version, preamble, inputmode, reservedbiasbits, l1scrambled, inband)

class dvbt2_freqinterleaver_cc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_freqinterleaver_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t") -> "gr::dtv::dvbt2_freqinterleaver_cc::sptr":
        r"""make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble) -> dvbt2_freqinterleaver_cc_sptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, preamble)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_freqinterleaver_cc

# Register dvbt2_freqinterleaver_cc in _dtv_swig1:
_dtv_swig1.dvbt2_freqinterleaver_cc_swigregister(dvbt2_freqinterleaver_cc)

def dvbt2_freqinterleaver_cc_make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t") -> "gr::dtv::dvbt2_freqinterleaver_cc::sptr":
    r"""dvbt2_freqinterleaver_cc_make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble) -> dvbt2_freqinterleaver_cc_sptr"""
    return _dtv_swig1.dvbt2_freqinterleaver_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, preamble)

class dvbt2_pilotgenerator_cc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_pilotgenerator_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t", misogroup: "gr::dtv::dvbt2_misogroup_t", equalization: "gr::dtv::dvbt2_equalization_t", bandwidth: "gr::dtv::dvbt2_bandwidth_t", vlength: "unsigned int") -> "gr::dtv::dvbt2_pilotgenerator_cc::sptr":
        r"""make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_misogroup_t misogroup, gr::dtv::dvbt2_equalization_t equalization, gr::dtv::dvbt2_bandwidth_t bandwidth, unsigned int vlength) -> dvbt2_pilotgenerator_cc_sptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, preamble, misogroup, equalization, bandwidth, vlength)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_pilotgenerator_cc

# Register dvbt2_pilotgenerator_cc in _dtv_swig1:
_dtv_swig1.dvbt2_pilotgenerator_cc_swigregister(dvbt2_pilotgenerator_cc)

def dvbt2_pilotgenerator_cc_make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t", misogroup: "gr::dtv::dvbt2_misogroup_t", equalization: "gr::dtv::dvbt2_equalization_t", bandwidth: "gr::dtv::dvbt2_bandwidth_t", vlength: "unsigned int") -> "gr::dtv::dvbt2_pilotgenerator_cc::sptr":
    r"""dvbt2_pilotgenerator_cc_make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_misogroup_t misogroup, gr::dtv::dvbt2_equalization_t equalization, gr::dtv::dvbt2_bandwidth_t bandwidth, unsigned int vlength) -> dvbt2_pilotgenerator_cc_sptr"""
    return _dtv_swig1.dvbt2_pilotgenerator_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, preamble, misogroup, equalization, bandwidth, vlength)

class dvbt2_paprtr_cc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_paprtr_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", vclip: "float", iterations: "int", vlength: "unsigned int") -> "gr::dtv::dvbt2_paprtr_cc::sptr":
        r"""make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, float vclip, int iterations, unsigned int vlength) -> dvbt2_paprtr_cc_sptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, vclip, iterations, vlength)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_paprtr_cc

# Register dvbt2_paprtr_cc in _dtv_swig1:
_dtv_swig1.dvbt2_paprtr_cc_swigregister(dvbt2_paprtr_cc)

def dvbt2_paprtr_cc_make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", vclip: "float", iterations: "int", vlength: "unsigned int") -> "gr::dtv::dvbt2_paprtr_cc::sptr":
    r"""dvbt2_paprtr_cc_make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, float vclip, int iterations, unsigned int vlength) -> dvbt2_paprtr_cc_sptr"""
    return _dtv_swig1.dvbt2_paprtr_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, vclip, iterations, vlength)

class dvbt2_p1insertion_cc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_p1insertion_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", preamble: "gr::dtv::dvbt2_preamble_t", showlevels: "gr::dtv::dvbt2_showlevels_t", vclip: "float") -> "gr::dtv::dvbt2_p1insertion_cc::sptr":
        r"""make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_showlevels_t showlevels, float vclip) -> dvbt2_p1insertion_cc_sptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_make(carriermode, fftsize, guardinterval, numdatasyms, preamble, showlevels, vclip)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_p1insertion_cc

# Register dvbt2_p1insertion_cc in _dtv_swig1:
_dtv_swig1.dvbt2_p1insertion_cc_swigregister(dvbt2_p1insertion_cc)

def dvbt2_p1insertion_cc_make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", preamble: "gr::dtv::dvbt2_preamble_t", showlevels: "gr::dtv::dvbt2_showlevels_t", vclip: "float") -> "gr::dtv::dvbt2_p1insertion_cc::sptr":
    r"""dvbt2_p1insertion_cc_make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_showlevels_t showlevels, float vclip) -> dvbt2_p1insertion_cc_sptr"""
    return _dtv_swig1.dvbt2_p1insertion_cc_make(carriermode, fftsize, guardinterval, numdatasyms, preamble, showlevels, vclip)

class dvbt2_miso_cc(object):
    r"""Proxy of C++ gr::dtv::dvbt2_miso_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t") -> "gr::dtv::dvbt2_miso_cc::sptr":
        r"""make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode) -> dvbt2_miso_cc_sptr"""
        return _dtv_swig1.dvbt2_miso_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_miso_cc

# Register dvbt2_miso_cc in _dtv_swig1:
_dtv_swig1.dvbt2_miso_cc_swigregister(dvbt2_miso_cc)

def dvbt2_miso_cc_make(carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t") -> "gr::dtv::dvbt2_miso_cc::sptr":
    r"""dvbt2_miso_cc_make(gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode) -> dvbt2_miso_cc_sptr"""
    return _dtv_swig1.dvbt2_miso_cc_make(carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode)

class dvbs2_interleaver_bb(object):
    r"""Proxy of C++ gr::dtv::dvbs2_interleaver_bb class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvbs2_interleaver_bb::sptr":
        r"""make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvbs2_interleaver_bb_sptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_make(framesize, rate, constellation)
    __swig_destroy__ = _dtv_swig1.delete_dvbs2_interleaver_bb

# Register dvbs2_interleaver_bb in _dtv_swig1:
_dtv_swig1.dvbs2_interleaver_bb_swigregister(dvbs2_interleaver_bb)

def dvbs2_interleaver_bb_make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvbs2_interleaver_bb::sptr":
    r"""dvbs2_interleaver_bb_make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvbs2_interleaver_bb_sptr"""
    return _dtv_swig1.dvbs2_interleaver_bb_make(framesize, rate, constellation)

class dvbs2_modulator_bc(object):
    r"""Proxy of C++ gr::dtv::dvbs2_modulator_bc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", interpolation: "gr::dtv::dvbs2_interpolation_t") -> "gr::dtv::dvbs2_modulator_bc::sptr":
        r"""make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbs2_interpolation_t interpolation) -> dvbs2_modulator_bc_sptr"""
        return _dtv_swig1.dvbs2_modulator_bc_make(framesize, rate, constellation, interpolation)
    __swig_destroy__ = _dtv_swig1.delete_dvbs2_modulator_bc

# Register dvbs2_modulator_bc in _dtv_swig1:
_dtv_swig1.dvbs2_modulator_bc_swigregister(dvbs2_modulator_bc)

def dvbs2_modulator_bc_make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", interpolation: "gr::dtv::dvbs2_interpolation_t") -> "gr::dtv::dvbs2_modulator_bc::sptr":
    r"""dvbs2_modulator_bc_make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbs2_interpolation_t interpolation) -> dvbs2_modulator_bc_sptr"""
    return _dtv_swig1.dvbs2_modulator_bc_make(framesize, rate, constellation, interpolation)

class dvbs2_physical_cc(object):
    r"""Proxy of C++ gr::dtv::dvbs2_physical_cc class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    @staticmethod
    def make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", pilots: "gr::dtv::dvbs2_pilots_t", goldcode: "int") -> "gr::dtv::dvbs2_physical_cc::sptr":
        r"""make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbs2_pilots_t pilots, int goldcode) -> dvbs2_physical_cc_sptr"""
        return _dtv_swig1.dvbs2_physical_cc_make(framesize, rate, constellation, pilots, goldcode)
    __swig_destroy__ = _dtv_swig1.delete_dvbs2_physical_cc

# Register dvbs2_physical_cc in _dtv_swig1:
_dtv_swig1.dvbs2_physical_cc_swigregister(dvbs2_physical_cc)

def dvbs2_physical_cc_make(framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", pilots: "gr::dtv::dvbs2_pilots_t", goldcode: "int") -> "gr::dtv::dvbs2_physical_cc::sptr":
    r"""dvbs2_physical_cc_make(gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbs2_pilots_t pilots, int goldcode) -> dvbs2_physical_cc_sptr"""
    return _dtv_swig1.dvbs2_physical_cc_make(framesize, rate, constellation, pilots, goldcode)

class dvb_bbheader_bb_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvb_bbheader_bb > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvb_bbheader_bb_sptr self) -> dvb_bbheader_bb_sptr
        __init__(dvb_bbheader_bb_sptr self, dvb_bbheader_bb p) -> dvb_bbheader_bb_sptr
        """
        _dtv_swig1.dvb_bbheader_bb_sptr_swiginit(self, _dtv_swig1.new_dvb_bbheader_bb_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvb_bbheader_bb *":
        r"""__deref__(dvb_bbheader_bb_sptr self) -> dvb_bbheader_bb"""
        return _dtv_swig1.dvb_bbheader_bb_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvb_bbheader_bb_sptr

    def make(self, standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", rolloff: "gr::dtv::dvbs2_rolloff_factor_t", mode: "gr::dtv::dvbt2_inputmode_t", inband: "gr::dtv::dvbt2_inband_t", fecblocks: "int", tsrate: "int") -> "gr::dtv::dvb_bbheader_bb::sptr":
        r"""make(dvb_bbheader_bb_sptr self, gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvbs2_rolloff_factor_t rolloff, gr::dtv::dvbt2_inputmode_t mode, gr::dtv::dvbt2_inband_t inband, int fecblocks, int tsrate) -> dvb_bbheader_bb_sptr"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_make(self, standard, framesize, rate, rolloff, mode, inband, fecblocks, tsrate)

    def history(self) -> "unsigned int":
        r"""history(dvb_bbheader_bb_sptr self) -> unsigned int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvb_bbheader_bb_sptr self, int which, int delay)
        declare_sample_delay(dvb_bbheader_bb_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvb_bbheader_bb_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvb_bbheader_bb_sptr self, int multiple)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvb_bbheader_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvb_bbheader_bb_sptr self) -> double"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvb_bbheader_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvb_bbheader_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvb_bbheader_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvb_bbheader_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvb_bbheader_bb_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvb_bbheader_bb_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvb_bbheader_bb_sptr self, std::string level)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvb_bbheader_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvb_bbheader_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvb_bbheader_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvb_bbheader_bb_sptr self)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvb_bbheader_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvb_bbheader_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvb_bbheader_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvb_bbheader_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvb_bbheader_bb_sptr self, long max_output_buffer)
        set_max_output_buffer(dvb_bbheader_bb_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvb_bbheader_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvb_bbheader_bb_sptr self, long min_output_buffer)
        set_min_output_buffer(dvb_bbheader_bb_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvb_bbheader_bb_sptr self, int which) -> float
        pc_input_buffers_full(dvb_bbheader_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvb_bbheader_bb_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvb_bbheader_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvb_bbheader_bb_sptr self, int which) -> float
        pc_input_buffers_full_var(dvb_bbheader_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvb_bbheader_bb_sptr self, int which) -> float
        pc_output_buffers_full(dvb_bbheader_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvb_bbheader_bb_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvb_bbheader_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvb_bbheader_bb_sptr self, int which) -> float
        pc_output_buffers_full_var(dvb_bbheader_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvb_bbheader_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvb_bbheader_bb_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvb_bbheader_bb_sptr self)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvb_bbheader_bb_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvb_bbheader_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvb_bbheader_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvb_bbheader_bb_sptr self, int priority) -> int"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvb_bbheader_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvb_bbheader_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvb_bbheader_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvb_bbheader_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvb_bbheader_bb_sptr self) -> long"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvb_bbheader_bb_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvb_bbheader_bb_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvb_bbheader_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvb_bbheader_bb_sptr self, std::string name)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvb_bbheader_bb_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvb_bbheader_bb_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvb_bbheader_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvb_bbheader_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvb_bbheader_bb_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bbheader_bb_sptr_message_subscribers(self, which_port)

# Register dvb_bbheader_bb_sptr in _dtv_swig1:
_dtv_swig1.dvb_bbheader_bb_sptr_swigregister(dvb_bbheader_bb_sptr)


dvb_bbheader_bb_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvb_bbheader_bb = dvb_bbheader_bb.make;

class dvb_bbscrambler_bb_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvb_bbscrambler_bb > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvb_bbscrambler_bb_sptr self) -> dvb_bbscrambler_bb_sptr
        __init__(dvb_bbscrambler_bb_sptr self, dvb_bbscrambler_bb p) -> dvb_bbscrambler_bb_sptr
        """
        _dtv_swig1.dvb_bbscrambler_bb_sptr_swiginit(self, _dtv_swig1.new_dvb_bbscrambler_bb_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvb_bbscrambler_bb *":
        r"""__deref__(dvb_bbscrambler_bb_sptr self) -> dvb_bbscrambler_bb"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvb_bbscrambler_bb_sptr

    def make(self, standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvb_bbscrambler_bb::sptr":
        r"""make(dvb_bbscrambler_bb_sptr self, gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate) -> dvb_bbscrambler_bb_sptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_make(self, standard, framesize, rate)

    def history(self) -> "unsigned int":
        r"""history(dvb_bbscrambler_bb_sptr self) -> unsigned int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvb_bbscrambler_bb_sptr self, int which, int delay)
        declare_sample_delay(dvb_bbscrambler_bb_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvb_bbscrambler_bb_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvb_bbscrambler_bb_sptr self, int multiple)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvb_bbscrambler_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvb_bbscrambler_bb_sptr self) -> double"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvb_bbscrambler_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvb_bbscrambler_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvb_bbscrambler_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvb_bbscrambler_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvb_bbscrambler_bb_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvb_bbscrambler_bb_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvb_bbscrambler_bb_sptr self, std::string level)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvb_bbscrambler_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvb_bbscrambler_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvb_bbscrambler_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvb_bbscrambler_bb_sptr self)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvb_bbscrambler_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvb_bbscrambler_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvb_bbscrambler_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvb_bbscrambler_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvb_bbscrambler_bb_sptr self, long max_output_buffer)
        set_max_output_buffer(dvb_bbscrambler_bb_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvb_bbscrambler_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvb_bbscrambler_bb_sptr self, long min_output_buffer)
        set_min_output_buffer(dvb_bbscrambler_bb_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvb_bbscrambler_bb_sptr self, int which) -> float
        pc_input_buffers_full(dvb_bbscrambler_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvb_bbscrambler_bb_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvb_bbscrambler_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvb_bbscrambler_bb_sptr self, int which) -> float
        pc_input_buffers_full_var(dvb_bbscrambler_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvb_bbscrambler_bb_sptr self, int which) -> float
        pc_output_buffers_full(dvb_bbscrambler_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvb_bbscrambler_bb_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvb_bbscrambler_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvb_bbscrambler_bb_sptr self, int which) -> float
        pc_output_buffers_full_var(dvb_bbscrambler_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvb_bbscrambler_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvb_bbscrambler_bb_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvb_bbscrambler_bb_sptr self)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvb_bbscrambler_bb_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvb_bbscrambler_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvb_bbscrambler_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvb_bbscrambler_bb_sptr self, int priority) -> int"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvb_bbscrambler_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvb_bbscrambler_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvb_bbscrambler_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvb_bbscrambler_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvb_bbscrambler_bb_sptr self) -> long"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvb_bbscrambler_bb_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvb_bbscrambler_bb_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvb_bbscrambler_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvb_bbscrambler_bb_sptr self, std::string name)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvb_bbscrambler_bb_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvb_bbscrambler_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvb_bbscrambler_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvb_bbscrambler_bb_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bbscrambler_bb_sptr_message_subscribers(self, which_port)

# Register dvb_bbscrambler_bb_sptr in _dtv_swig1:
_dtv_swig1.dvb_bbscrambler_bb_sptr_swigregister(dvb_bbscrambler_bb_sptr)


dvb_bbscrambler_bb_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvb_bbscrambler_bb = dvb_bbscrambler_bb.make;

class dvb_bch_bb_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvb_bch_bb > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvb_bch_bb_sptr self) -> dvb_bch_bb_sptr
        __init__(dvb_bch_bb_sptr self, dvb_bch_bb p) -> dvb_bch_bb_sptr
        """
        _dtv_swig1.dvb_bch_bb_sptr_swiginit(self, _dtv_swig1.new_dvb_bch_bb_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvb_bch_bb *":
        r"""__deref__(dvb_bch_bb_sptr self) -> dvb_bch_bb"""
        return _dtv_swig1.dvb_bch_bb_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvb_bch_bb_sptr

    def make(self, standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t") -> "gr::dtv::dvb_bch_bb::sptr":
        r"""make(dvb_bch_bb_sptr self, gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate) -> dvb_bch_bb_sptr"""
        return _dtv_swig1.dvb_bch_bb_sptr_make(self, standard, framesize, rate)

    def history(self) -> "unsigned int":
        r"""history(dvb_bch_bb_sptr self) -> unsigned int"""
        return _dtv_swig1.dvb_bch_bb_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvb_bch_bb_sptr self, int which, int delay)
        declare_sample_delay(dvb_bch_bb_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvb_bch_bb_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvb_bch_bb_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvb_bch_bb_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvb_bch_bb_sptr self, int multiple)"""
        return _dtv_swig1.dvb_bch_bb_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvb_bch_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bch_bb_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvb_bch_bb_sptr self) -> double"""
        return _dtv_swig1.dvb_bch_bb_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvb_bch_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_bch_bb_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvb_bch_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_bch_bb_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvb_bch_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bch_bb_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvb_bch_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bch_bb_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvb_bch_bb_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvb_bch_bb_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvb_bch_bb_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvb_bch_bb_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvb_bch_bb_sptr self, std::string level)"""
        return _dtv_swig1.dvb_bch_bb_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvb_bch_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bch_bb_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvb_bch_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bch_bb_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvb_bch_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_bch_bb_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvb_bch_bb_sptr self)"""
        return _dtv_swig1.dvb_bch_bb_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvb_bch_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_bch_bb_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvb_bch_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_bch_bb_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvb_bch_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bch_bb_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvb_bch_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_bch_bb_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvb_bch_bb_sptr self, long max_output_buffer)
        set_max_output_buffer(dvb_bch_bb_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvb_bch_bb_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvb_bch_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_bch_bb_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvb_bch_bb_sptr self, long min_output_buffer)
        set_min_output_buffer(dvb_bch_bb_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvb_bch_bb_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvb_bch_bb_sptr self, int which) -> float
        pc_input_buffers_full(dvb_bch_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bch_bb_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvb_bch_bb_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvb_bch_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bch_bb_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvb_bch_bb_sptr self, int which) -> float
        pc_input_buffers_full_var(dvb_bch_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bch_bb_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvb_bch_bb_sptr self, int which) -> float
        pc_output_buffers_full(dvb_bch_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bch_bb_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvb_bch_bb_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvb_bch_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bch_bb_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvb_bch_bb_sptr self, int which) -> float
        pc_output_buffers_full_var(dvb_bch_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_bch_bb_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvb_bch_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_bch_bb_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvb_bch_bb_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvb_bch_bb_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvb_bch_bb_sptr self)"""
        return _dtv_swig1.dvb_bch_bb_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvb_bch_bb_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvb_bch_bb_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvb_bch_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bch_bb_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvb_bch_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_bch_bb_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvb_bch_bb_sptr self, int priority) -> int"""
        return _dtv_swig1.dvb_bch_bb_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvb_bch_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bch_bb_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvb_bch_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bch_bb_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvb_bch_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_bch_bb_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvb_bch_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_bch_bb_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvb_bch_bb_sptr self) -> long"""
        return _dtv_swig1.dvb_bch_bb_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvb_bch_bb_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvb_bch_bb_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvb_bch_bb_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvb_bch_bb_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvb_bch_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_bch_bb_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvb_bch_bb_sptr self, std::string name)"""
        return _dtv_swig1.dvb_bch_bb_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvb_bch_bb_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvb_bch_bb_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvb_bch_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bch_bb_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvb_bch_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bch_bb_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvb_bch_bb_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_bch_bb_sptr_message_subscribers(self, which_port)

# Register dvb_bch_bb_sptr in _dtv_swig1:
_dtv_swig1.dvb_bch_bb_sptr_swigregister(dvb_bch_bb_sptr)


dvb_bch_bb_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvb_bch_bb = dvb_bch_bb.make;

class dvb_ldpc_bb_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvb_ldpc_bb > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvb_ldpc_bb_sptr self) -> dvb_ldpc_bb_sptr
        __init__(dvb_ldpc_bb_sptr self, dvb_ldpc_bb p) -> dvb_ldpc_bb_sptr
        """
        _dtv_swig1.dvb_ldpc_bb_sptr_swiginit(self, _dtv_swig1.new_dvb_ldpc_bb_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvb_ldpc_bb *":
        r"""__deref__(dvb_ldpc_bb_sptr self) -> dvb_ldpc_bb"""
        return _dtv_swig1.dvb_ldpc_bb_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvb_ldpc_bb_sptr

    def make(self, standard: "gr::dtv::dvb_standard_t", framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvb_ldpc_bb::sptr":
        r"""make(dvb_ldpc_bb_sptr self, gr::dtv::dvb_standard_t standard, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvb_ldpc_bb_sptr"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_make(self, standard, framesize, rate, constellation)

    def history(self) -> "unsigned int":
        r"""history(dvb_ldpc_bb_sptr self) -> unsigned int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvb_ldpc_bb_sptr self, int which, int delay)
        declare_sample_delay(dvb_ldpc_bb_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvb_ldpc_bb_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvb_ldpc_bb_sptr self, int multiple)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvb_ldpc_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvb_ldpc_bb_sptr self) -> double"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvb_ldpc_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvb_ldpc_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvb_ldpc_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvb_ldpc_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvb_ldpc_bb_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvb_ldpc_bb_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvb_ldpc_bb_sptr self, std::string level)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvb_ldpc_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvb_ldpc_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvb_ldpc_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvb_ldpc_bb_sptr self)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvb_ldpc_bb_sptr self) -> bool"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvb_ldpc_bb_sptr self, int m)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvb_ldpc_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvb_ldpc_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvb_ldpc_bb_sptr self, long max_output_buffer)
        set_max_output_buffer(dvb_ldpc_bb_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvb_ldpc_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvb_ldpc_bb_sptr self, long min_output_buffer)
        set_min_output_buffer(dvb_ldpc_bb_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvb_ldpc_bb_sptr self, int which) -> float
        pc_input_buffers_full(dvb_ldpc_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvb_ldpc_bb_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvb_ldpc_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvb_ldpc_bb_sptr self, int which) -> float
        pc_input_buffers_full_var(dvb_ldpc_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvb_ldpc_bb_sptr self, int which) -> float
        pc_output_buffers_full(dvb_ldpc_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvb_ldpc_bb_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvb_ldpc_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvb_ldpc_bb_sptr self, int which) -> float
        pc_output_buffers_full_var(dvb_ldpc_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvb_ldpc_bb_sptr self) -> float"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvb_ldpc_bb_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvb_ldpc_bb_sptr self)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvb_ldpc_bb_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvb_ldpc_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvb_ldpc_bb_sptr self) -> int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvb_ldpc_bb_sptr self, int priority) -> int"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvb_ldpc_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvb_ldpc_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvb_ldpc_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvb_ldpc_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvb_ldpc_bb_sptr self) -> long"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvb_ldpc_bb_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvb_ldpc_bb_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvb_ldpc_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvb_ldpc_bb_sptr self, std::string name)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvb_ldpc_bb_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvb_ldpc_bb_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvb_ldpc_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvb_ldpc_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvb_ldpc_bb_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvb_ldpc_bb_sptr_message_subscribers(self, which_port)

# Register dvb_ldpc_bb_sptr in _dtv_swig1:
_dtv_swig1.dvb_ldpc_bb_sptr_swigregister(dvb_ldpc_bb_sptr)


dvb_ldpc_bb_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvb_ldpc_bb = dvb_ldpc_bb.make;

class dvbt2_interleaver_bb_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_interleaver_bb > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_interleaver_bb_sptr self) -> dvbt2_interleaver_bb_sptr
        __init__(dvbt2_interleaver_bb_sptr self, dvbt2_interleaver_bb p) -> dvbt2_interleaver_bb_sptr
        """
        _dtv_swig1.dvbt2_interleaver_bb_sptr_swiginit(self, _dtv_swig1.new_dvbt2_interleaver_bb_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_interleaver_bb *":
        r"""__deref__(dvbt2_interleaver_bb_sptr self) -> dvbt2_interleaver_bb"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_interleaver_bb_sptr

    def make(self, framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvbt2_interleaver_bb::sptr":
        r"""make(dvbt2_interleaver_bb_sptr self, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvbt2_interleaver_bb_sptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_make(self, framesize, rate, constellation)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_interleaver_bb_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_interleaver_bb_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_interleaver_bb_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_interleaver_bb_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_interleaver_bb_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_interleaver_bb_sptr self) -> double"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_interleaver_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_interleaver_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_interleaver_bb_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_interleaver_bb_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_interleaver_bb_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_interleaver_bb_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_interleaver_bb_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_interleaver_bb_sptr self, int m)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_interleaver_bb_sptr self)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_interleaver_bb_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_interleaver_bb_sptr self, int m)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_interleaver_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_interleaver_bb_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_interleaver_bb_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_interleaver_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_interleaver_bb_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_interleaver_bb_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_interleaver_bb_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_interleaver_bb_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_interleaver_bb_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_interleaver_bb_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_interleaver_bb_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_interleaver_bb_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_interleaver_bb_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_interleaver_bb_sptr self)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_interleaver_bb_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_interleaver_bb_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_interleaver_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_interleaver_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_interleaver_bb_sptr self) -> long"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_interleaver_bb_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_interleaver_bb_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_interleaver_bb_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_interleaver_bb_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_interleaver_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_interleaver_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_interleaver_bb_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_interleaver_bb_sptr_message_subscribers(self, which_port)

# Register dvbt2_interleaver_bb_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_interleaver_bb_sptr_swigregister(dvbt2_interleaver_bb_sptr)


dvbt2_interleaver_bb_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_interleaver_bb = dvbt2_interleaver_bb.make;

class dvbt2_modulator_bc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_modulator_bc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_modulator_bc_sptr self) -> dvbt2_modulator_bc_sptr
        __init__(dvbt2_modulator_bc_sptr self, dvbt2_modulator_bc p) -> dvbt2_modulator_bc_sptr
        """
        _dtv_swig1.dvbt2_modulator_bc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_modulator_bc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_modulator_bc *":
        r"""__deref__(dvbt2_modulator_bc_sptr self) -> dvbt2_modulator_bc"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_modulator_bc_sptr

    def make(self, framesize: "gr::dtv::dvb_framesize_t", constellation: "gr::dtv::dvb_constellation_t", rotation: "gr::dtv::dvbt2_rotation_t") -> "gr::dtv::dvbt2_modulator_bc::sptr":
        r"""make(dvbt2_modulator_bc_sptr self, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt2_rotation_t rotation) -> dvbt2_modulator_bc_sptr"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_make(self, framesize, constellation, rotation)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_modulator_bc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_modulator_bc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_modulator_bc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_modulator_bc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_modulator_bc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_modulator_bc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_modulator_bc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_modulator_bc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_modulator_bc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_modulator_bc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_modulator_bc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_modulator_bc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_modulator_bc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_modulator_bc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_modulator_bc_sptr self)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_modulator_bc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_modulator_bc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_modulator_bc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_modulator_bc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_modulator_bc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_modulator_bc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_modulator_bc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_modulator_bc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_modulator_bc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_modulator_bc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_modulator_bc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_modulator_bc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_modulator_bc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_modulator_bc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_modulator_bc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_modulator_bc_sptr self)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_modulator_bc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_modulator_bc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_modulator_bc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_modulator_bc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_modulator_bc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_modulator_bc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_modulator_bc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_modulator_bc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_modulator_bc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_modulator_bc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_modulator_bc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_modulator_bc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_modulator_bc_sptr_message_subscribers(self, which_port)

# Register dvbt2_modulator_bc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_modulator_bc_sptr_swigregister(dvbt2_modulator_bc_sptr)


dvbt2_modulator_bc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_modulator_bc = dvbt2_modulator_bc.make;

class dvbt2_cellinterleaver_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_cellinterleaver_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_cellinterleaver_cc_sptr self) -> dvbt2_cellinterleaver_cc_sptr
        __init__(dvbt2_cellinterleaver_cc_sptr self, dvbt2_cellinterleaver_cc p) -> dvbt2_cellinterleaver_cc_sptr
        """
        _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_cellinterleaver_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_cellinterleaver_cc *":
        r"""__deref__(dvbt2_cellinterleaver_cc_sptr self) -> dvbt2_cellinterleaver_cc"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_cellinterleaver_cc_sptr

    def make(self, framesize: "gr::dtv::dvb_framesize_t", constellation: "gr::dtv::dvb_constellation_t", fecblocks: "int", tiblocks: "int") -> "gr::dtv::dvbt2_cellinterleaver_cc::sptr":
        r"""make(dvbt2_cellinterleaver_cc_sptr self, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_constellation_t constellation, int fecblocks, int tiblocks) -> dvbt2_cellinterleaver_cc_sptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_make(self, framesize, constellation, fecblocks, tiblocks)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_cellinterleaver_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_cellinterleaver_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_cellinterleaver_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_cellinterleaver_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_cellinterleaver_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_cellinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_cellinterleaver_cc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_cellinterleaver_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_cellinterleaver_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_cellinterleaver_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_cellinterleaver_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_cellinterleaver_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_cellinterleaver_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_cellinterleaver_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_cellinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_cellinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_cellinterleaver_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_cellinterleaver_cc_sptr self)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_cellinterleaver_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_cellinterleaver_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_cellinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_cellinterleaver_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_cellinterleaver_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_cellinterleaver_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_cellinterleaver_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_cellinterleaver_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_cellinterleaver_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_cellinterleaver_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_cellinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_cellinterleaver_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_cellinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_cellinterleaver_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_cellinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_cellinterleaver_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_cellinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_cellinterleaver_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_cellinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_cellinterleaver_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_cellinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_cellinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_cellinterleaver_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_cellinterleaver_cc_sptr self)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_cellinterleaver_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_cellinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_cellinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_cellinterleaver_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_cellinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_cellinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_cellinterleaver_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_cellinterleaver_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_cellinterleaver_cc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_cellinterleaver_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_cellinterleaver_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_cellinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_cellinterleaver_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_cellinterleaver_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_cellinterleaver_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_cellinterleaver_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_cellinterleaver_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_cellinterleaver_cc_sptr_message_subscribers(self, which_port)

# Register dvbt2_cellinterleaver_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_cellinterleaver_cc_sptr_swigregister(dvbt2_cellinterleaver_cc_sptr)


dvbt2_cellinterleaver_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_cellinterleaver_cc = dvbt2_cellinterleaver_cc.make;

class dvbt2_framemapper_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_framemapper_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_framemapper_cc_sptr self) -> dvbt2_framemapper_cc_sptr
        __init__(dvbt2_framemapper_cc_sptr self, dvbt2_framemapper_cc p) -> dvbt2_framemapper_cc_sptr
        """
        _dtv_swig1.dvbt2_framemapper_cc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_framemapper_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_framemapper_cc *":
        r"""__deref__(dvbt2_framemapper_cc_sptr self) -> dvbt2_framemapper_cc"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_framemapper_cc_sptr

    def make(self, framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", rotation: "gr::dtv::dvbt2_rotation_t", fecblocks: "int", tiblocks: "int", carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", guardinterval: "gr::dtv::dvb_guardinterval_t", l1constellation: "gr::dtv::dvbt2_l1constellation_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", t2frames: "int", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t", inputmode: "gr::dtv::dvbt2_inputmode_t", reservedbiasbits: "gr::dtv::dvbt2_reservedbiasbits_t", l1scrambled: "gr::dtv::dvbt2_l1scrambled_t", inband: "gr::dtv::dvbt2_inband_t") -> "gr::dtv::dvbt2_framemapper_cc::sptr":
        r"""make(dvbt2_framemapper_cc_sptr self, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbt2_rotation_t rotation, int fecblocks, int tiblocks, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvb_guardinterval_t guardinterval, gr::dtv::dvbt2_l1constellation_t l1constellation, gr::dtv::dvbt2_pilotpattern_t pilotpattern, int t2frames, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_inputmode_t inputmode, gr::dtv::dvbt2_reservedbiasbits_t reservedbiasbits, gr::dtv::dvbt2_l1scrambled_t l1scrambled, gr::dtv::dvbt2_inband_t inband) -> dvbt2_framemapper_cc_sptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_make(self, framesize, rate, constellation, rotation, fecblocks, tiblocks, carriermode, fftsize, guardinterval, l1constellation, pilotpattern, t2frames, numdatasyms, paprmode, version, preamble, inputmode, reservedbiasbits, l1scrambled, inband)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_framemapper_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_framemapper_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_framemapper_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_framemapper_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_framemapper_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_framemapper_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_framemapper_cc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_framemapper_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_framemapper_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_framemapper_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_framemapper_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_framemapper_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_framemapper_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_framemapper_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_framemapper_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_framemapper_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_framemapper_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_framemapper_cc_sptr self)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_framemapper_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_framemapper_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_framemapper_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_framemapper_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_framemapper_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_framemapper_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_framemapper_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_framemapper_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_framemapper_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_framemapper_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_framemapper_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_framemapper_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_framemapper_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_framemapper_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_framemapper_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_framemapper_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_framemapper_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_framemapper_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_framemapper_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_framemapper_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_framemapper_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_framemapper_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_framemapper_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_framemapper_cc_sptr self)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_framemapper_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_framemapper_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_framemapper_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_framemapper_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_framemapper_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_framemapper_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_framemapper_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_framemapper_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_framemapper_cc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_framemapper_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_framemapper_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_framemapper_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_framemapper_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_framemapper_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_framemapper_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_framemapper_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_framemapper_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_framemapper_cc_sptr_message_subscribers(self, which_port)

# Register dvbt2_framemapper_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_framemapper_cc_sptr_swigregister(dvbt2_framemapper_cc_sptr)


dvbt2_framemapper_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_framemapper_cc = dvbt2_framemapper_cc.make;

class dvbt2_freqinterleaver_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_freqinterleaver_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_freqinterleaver_cc_sptr self) -> dvbt2_freqinterleaver_cc_sptr
        __init__(dvbt2_freqinterleaver_cc_sptr self, dvbt2_freqinterleaver_cc p) -> dvbt2_freqinterleaver_cc_sptr
        """
        _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_freqinterleaver_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_freqinterleaver_cc *":
        r"""__deref__(dvbt2_freqinterleaver_cc_sptr self) -> dvbt2_freqinterleaver_cc"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_freqinterleaver_cc_sptr

    def make(self, carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t") -> "gr::dtv::dvbt2_freqinterleaver_cc::sptr":
        r"""make(dvbt2_freqinterleaver_cc_sptr self, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble) -> dvbt2_freqinterleaver_cc_sptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_make(self, carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, preamble)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_freqinterleaver_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_freqinterleaver_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_freqinterleaver_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_freqinterleaver_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_freqinterleaver_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_freqinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_freqinterleaver_cc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_freqinterleaver_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_freqinterleaver_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_freqinterleaver_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_freqinterleaver_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_freqinterleaver_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_freqinterleaver_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_freqinterleaver_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_freqinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_freqinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_freqinterleaver_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_freqinterleaver_cc_sptr self)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_freqinterleaver_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_freqinterleaver_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_freqinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_freqinterleaver_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_freqinterleaver_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_freqinterleaver_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_freqinterleaver_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_freqinterleaver_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_freqinterleaver_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_freqinterleaver_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_freqinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_freqinterleaver_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_freqinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_freqinterleaver_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_freqinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_freqinterleaver_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_freqinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_freqinterleaver_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_freqinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_freqinterleaver_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_freqinterleaver_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_freqinterleaver_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_freqinterleaver_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_freqinterleaver_cc_sptr self)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_freqinterleaver_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_freqinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_freqinterleaver_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_freqinterleaver_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_freqinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_freqinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_freqinterleaver_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_freqinterleaver_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_freqinterleaver_cc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_freqinterleaver_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_freqinterleaver_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_freqinterleaver_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_freqinterleaver_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_freqinterleaver_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_freqinterleaver_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_freqinterleaver_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_freqinterleaver_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_freqinterleaver_cc_sptr_message_subscribers(self, which_port)

# Register dvbt2_freqinterleaver_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_freqinterleaver_cc_sptr_swigregister(dvbt2_freqinterleaver_cc_sptr)


dvbt2_freqinterleaver_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_freqinterleaver_cc = dvbt2_freqinterleaver_cc.make;

class dvbt2_pilotgenerator_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_pilotgenerator_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_pilotgenerator_cc_sptr self) -> dvbt2_pilotgenerator_cc_sptr
        __init__(dvbt2_pilotgenerator_cc_sptr self, dvbt2_pilotgenerator_cc p) -> dvbt2_pilotgenerator_cc_sptr
        """
        _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_pilotgenerator_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_pilotgenerator_cc *":
        r"""__deref__(dvbt2_pilotgenerator_cc_sptr self) -> dvbt2_pilotgenerator_cc"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_pilotgenerator_cc_sptr

    def make(self, carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", preamble: "gr::dtv::dvbt2_preamble_t", misogroup: "gr::dtv::dvbt2_misogroup_t", equalization: "gr::dtv::dvbt2_equalization_t", bandwidth: "gr::dtv::dvbt2_bandwidth_t", vlength: "unsigned int") -> "gr::dtv::dvbt2_pilotgenerator_cc::sptr":
        r"""make(dvbt2_pilotgenerator_cc_sptr self, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_misogroup_t misogroup, gr::dtv::dvbt2_equalization_t equalization, gr::dtv::dvbt2_bandwidth_t bandwidth, unsigned int vlength) -> dvbt2_pilotgenerator_cc_sptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_make(self, carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, preamble, misogroup, equalization, bandwidth, vlength)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_pilotgenerator_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_pilotgenerator_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_pilotgenerator_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_pilotgenerator_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_pilotgenerator_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_pilotgenerator_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_pilotgenerator_cc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_pilotgenerator_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_pilotgenerator_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_pilotgenerator_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_pilotgenerator_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_pilotgenerator_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_pilotgenerator_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_pilotgenerator_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_pilotgenerator_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_pilotgenerator_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_pilotgenerator_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_pilotgenerator_cc_sptr self)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_pilotgenerator_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_pilotgenerator_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_pilotgenerator_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_pilotgenerator_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_pilotgenerator_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_pilotgenerator_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_pilotgenerator_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_pilotgenerator_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_pilotgenerator_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_pilotgenerator_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_pilotgenerator_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_pilotgenerator_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_pilotgenerator_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_pilotgenerator_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_pilotgenerator_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_pilotgenerator_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_pilotgenerator_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_pilotgenerator_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_pilotgenerator_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_pilotgenerator_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_pilotgenerator_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_pilotgenerator_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_pilotgenerator_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_pilotgenerator_cc_sptr self)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_pilotgenerator_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_pilotgenerator_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_pilotgenerator_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_pilotgenerator_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_pilotgenerator_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_pilotgenerator_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_pilotgenerator_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_pilotgenerator_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_pilotgenerator_cc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_pilotgenerator_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_pilotgenerator_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_pilotgenerator_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_pilotgenerator_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_pilotgenerator_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_pilotgenerator_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_pilotgenerator_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_pilotgenerator_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_pilotgenerator_cc_sptr_message_subscribers(self, which_port)

# Register dvbt2_pilotgenerator_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_pilotgenerator_cc_sptr_swigregister(dvbt2_pilotgenerator_cc_sptr)


dvbt2_pilotgenerator_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_pilotgenerator_cc = dvbt2_pilotgenerator_cc.make;

class dvbt2_paprtr_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_paprtr_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_paprtr_cc_sptr self) -> dvbt2_paprtr_cc_sptr
        __init__(dvbt2_paprtr_cc_sptr self, dvbt2_paprtr_cc p) -> dvbt2_paprtr_cc_sptr
        """
        _dtv_swig1.dvbt2_paprtr_cc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_paprtr_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_paprtr_cc *":
        r"""__deref__(dvbt2_paprtr_cc_sptr self) -> dvbt2_paprtr_cc"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_paprtr_cc_sptr

    def make(self, carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t", version: "gr::dtv::dvbt2_version_t", vclip: "float", iterations: "int", vlength: "unsigned int") -> "gr::dtv::dvbt2_paprtr_cc::sptr":
        r"""make(dvbt2_paprtr_cc_sptr self, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode, gr::dtv::dvbt2_version_t version, float vclip, int iterations, unsigned int vlength) -> dvbt2_paprtr_cc_sptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_make(self, carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode, version, vclip, iterations, vlength)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_paprtr_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_paprtr_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_paprtr_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_paprtr_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_paprtr_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_paprtr_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_paprtr_cc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_paprtr_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_paprtr_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_paprtr_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_paprtr_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_paprtr_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_paprtr_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_paprtr_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_paprtr_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_paprtr_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_paprtr_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_paprtr_cc_sptr self)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_paprtr_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_paprtr_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_paprtr_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_paprtr_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_paprtr_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_paprtr_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_paprtr_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_paprtr_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_paprtr_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_paprtr_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_paprtr_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_paprtr_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_paprtr_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_paprtr_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_paprtr_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_paprtr_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_paprtr_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_paprtr_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_paprtr_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_paprtr_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_paprtr_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_paprtr_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_paprtr_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_paprtr_cc_sptr self)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_paprtr_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_paprtr_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_paprtr_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_paprtr_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_paprtr_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_paprtr_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_paprtr_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_paprtr_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_paprtr_cc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_paprtr_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_paprtr_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_paprtr_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_paprtr_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_paprtr_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_paprtr_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_paprtr_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_paprtr_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_paprtr_cc_sptr_message_subscribers(self, which_port)

# Register dvbt2_paprtr_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_paprtr_cc_sptr_swigregister(dvbt2_paprtr_cc_sptr)


dvbt2_paprtr_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_paprtr_cc = dvbt2_paprtr_cc.make;

class dvbt2_p1insertion_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_p1insertion_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_p1insertion_cc_sptr self) -> dvbt2_p1insertion_cc_sptr
        __init__(dvbt2_p1insertion_cc_sptr self, dvbt2_p1insertion_cc p) -> dvbt2_p1insertion_cc_sptr
        """
        _dtv_swig1.dvbt2_p1insertion_cc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_p1insertion_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_p1insertion_cc *":
        r"""__deref__(dvbt2_p1insertion_cc_sptr self) -> dvbt2_p1insertion_cc"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_p1insertion_cc_sptr

    def make(self, carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", preamble: "gr::dtv::dvbt2_preamble_t", showlevels: "gr::dtv::dvbt2_showlevels_t", vclip: "float") -> "gr::dtv::dvbt2_p1insertion_cc::sptr":
        r"""make(dvbt2_p1insertion_cc_sptr self, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_preamble_t preamble, gr::dtv::dvbt2_showlevels_t showlevels, float vclip) -> dvbt2_p1insertion_cc_sptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_make(self, carriermode, fftsize, guardinterval, numdatasyms, preamble, showlevels, vclip)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_p1insertion_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_p1insertion_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_p1insertion_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_p1insertion_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_p1insertion_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_p1insertion_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_p1insertion_cc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_p1insertion_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_p1insertion_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_p1insertion_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_p1insertion_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_p1insertion_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_p1insertion_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_p1insertion_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_p1insertion_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_p1insertion_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_p1insertion_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_p1insertion_cc_sptr self)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_p1insertion_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_p1insertion_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_p1insertion_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_p1insertion_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_p1insertion_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_p1insertion_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_p1insertion_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_p1insertion_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_p1insertion_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_p1insertion_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_p1insertion_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_p1insertion_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_p1insertion_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_p1insertion_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_p1insertion_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_p1insertion_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_p1insertion_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_p1insertion_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_p1insertion_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_p1insertion_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_p1insertion_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_p1insertion_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_p1insertion_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_p1insertion_cc_sptr self)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_p1insertion_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_p1insertion_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_p1insertion_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_p1insertion_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_p1insertion_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_p1insertion_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_p1insertion_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_p1insertion_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_p1insertion_cc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_p1insertion_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_p1insertion_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_p1insertion_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_p1insertion_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_p1insertion_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_p1insertion_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_p1insertion_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_p1insertion_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_p1insertion_cc_sptr_message_subscribers(self, which_port)

# Register dvbt2_p1insertion_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_p1insertion_cc_sptr_swigregister(dvbt2_p1insertion_cc_sptr)


dvbt2_p1insertion_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_p1insertion_cc = dvbt2_p1insertion_cc.make;

class dvbt2_miso_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbt2_miso_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbt2_miso_cc_sptr self) -> dvbt2_miso_cc_sptr
        __init__(dvbt2_miso_cc_sptr self, dvbt2_miso_cc p) -> dvbt2_miso_cc_sptr
        """
        _dtv_swig1.dvbt2_miso_cc_sptr_swiginit(self, _dtv_swig1.new_dvbt2_miso_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbt2_miso_cc *":
        r"""__deref__(dvbt2_miso_cc_sptr self) -> dvbt2_miso_cc"""
        return _dtv_swig1.dvbt2_miso_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbt2_miso_cc_sptr

    def make(self, carriermode: "gr::dtv::dvbt2_extended_carrier_t", fftsize: "gr::dtv::dvbt2_fftsize_t", pilotpattern: "gr::dtv::dvbt2_pilotpattern_t", guardinterval: "gr::dtv::dvb_guardinterval_t", numdatasyms: "int", paprmode: "gr::dtv::dvbt2_papr_t") -> "gr::dtv::dvbt2_miso_cc::sptr":
        r"""make(dvbt2_miso_cc_sptr self, gr::dtv::dvbt2_extended_carrier_t carriermode, gr::dtv::dvbt2_fftsize_t fftsize, gr::dtv::dvbt2_pilotpattern_t pilotpattern, gr::dtv::dvb_guardinterval_t guardinterval, int numdatasyms, gr::dtv::dvbt2_papr_t paprmode) -> dvbt2_miso_cc_sptr"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_make(self, carriermode, fftsize, pilotpattern, guardinterval, numdatasyms, paprmode)

    def history(self) -> "unsigned int":
        r"""history(dvbt2_miso_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbt2_miso_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbt2_miso_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbt2_miso_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbt2_miso_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbt2_miso_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbt2_miso_cc_sptr self) -> double"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbt2_miso_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbt2_miso_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbt2_miso_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbt2_miso_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbt2_miso_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbt2_miso_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbt2_miso_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbt2_miso_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbt2_miso_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbt2_miso_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbt2_miso_cc_sptr self)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbt2_miso_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbt2_miso_cc_sptr self, int m)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbt2_miso_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbt2_miso_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbt2_miso_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbt2_miso_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbt2_miso_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbt2_miso_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbt2_miso_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbt2_miso_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbt2_miso_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbt2_miso_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbt2_miso_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbt2_miso_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbt2_miso_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbt2_miso_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbt2_miso_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbt2_miso_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbt2_miso_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbt2_miso_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbt2_miso_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbt2_miso_cc_sptr self) -> float"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbt2_miso_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbt2_miso_cc_sptr self)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbt2_miso_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbt2_miso_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbt2_miso_cc_sptr self) -> int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbt2_miso_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbt2_miso_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbt2_miso_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbt2_miso_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbt2_miso_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbt2_miso_cc_sptr self) -> long"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbt2_miso_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbt2_miso_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbt2_miso_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbt2_miso_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbt2_miso_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbt2_miso_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbt2_miso_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbt2_miso_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbt2_miso_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbt2_miso_cc_sptr_message_subscribers(self, which_port)

# Register dvbt2_miso_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbt2_miso_cc_sptr_swigregister(dvbt2_miso_cc_sptr)


dvbt2_miso_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbt2_miso_cc = dvbt2_miso_cc.make;

class dvbs2_interleaver_bb_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbs2_interleaver_bb > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbs2_interleaver_bb_sptr self) -> dvbs2_interleaver_bb_sptr
        __init__(dvbs2_interleaver_bb_sptr self, dvbs2_interleaver_bb p) -> dvbs2_interleaver_bb_sptr
        """
        _dtv_swig1.dvbs2_interleaver_bb_sptr_swiginit(self, _dtv_swig1.new_dvbs2_interleaver_bb_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbs2_interleaver_bb *":
        r"""__deref__(dvbs2_interleaver_bb_sptr self) -> dvbs2_interleaver_bb"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbs2_interleaver_bb_sptr

    def make(self, framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t") -> "gr::dtv::dvbs2_interleaver_bb::sptr":
        r"""make(dvbs2_interleaver_bb_sptr self, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation) -> dvbs2_interleaver_bb_sptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_make(self, framesize, rate, constellation)

    def history(self) -> "unsigned int":
        r"""history(dvbs2_interleaver_bb_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbs2_interleaver_bb_sptr self, int which, int delay)
        declare_sample_delay(dvbs2_interleaver_bb_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbs2_interleaver_bb_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbs2_interleaver_bb_sptr self, int multiple)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbs2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbs2_interleaver_bb_sptr self) -> double"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbs2_interleaver_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbs2_interleaver_bb_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbs2_interleaver_bb_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbs2_interleaver_bb_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbs2_interleaver_bb_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbs2_interleaver_bb_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbs2_interleaver_bb_sptr self, std::string level)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbs2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbs2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbs2_interleaver_bb_sptr self, int m)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbs2_interleaver_bb_sptr self)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbs2_interleaver_bb_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbs2_interleaver_bb_sptr self, int m)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbs2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbs2_interleaver_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbs2_interleaver_bb_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbs2_interleaver_bb_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbs2_interleaver_bb_sptr self, int i) -> long"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbs2_interleaver_bb_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbs2_interleaver_bb_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbs2_interleaver_bb_sptr self, int which) -> float
        pc_input_buffers_full(dvbs2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbs2_interleaver_bb_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbs2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbs2_interleaver_bb_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbs2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbs2_interleaver_bb_sptr self, int which) -> float
        pc_output_buffers_full(dvbs2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbs2_interleaver_bb_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbs2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbs2_interleaver_bb_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbs2_interleaver_bb_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbs2_interleaver_bb_sptr self) -> float"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbs2_interleaver_bb_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbs2_interleaver_bb_sptr self)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbs2_interleaver_bb_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbs2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbs2_interleaver_bb_sptr self) -> int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbs2_interleaver_bb_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbs2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbs2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbs2_interleaver_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbs2_interleaver_bb_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbs2_interleaver_bb_sptr self) -> long"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbs2_interleaver_bb_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbs2_interleaver_bb_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbs2_interleaver_bb_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbs2_interleaver_bb_sptr self, std::string name)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbs2_interleaver_bb_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbs2_interleaver_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbs2_interleaver_bb_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbs2_interleaver_bb_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_interleaver_bb_sptr_message_subscribers(self, which_port)

# Register dvbs2_interleaver_bb_sptr in _dtv_swig1:
_dtv_swig1.dvbs2_interleaver_bb_sptr_swigregister(dvbs2_interleaver_bb_sptr)


dvbs2_interleaver_bb_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbs2_interleaver_bb = dvbs2_interleaver_bb.make;

class dvbs2_modulator_bc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbs2_modulator_bc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbs2_modulator_bc_sptr self) -> dvbs2_modulator_bc_sptr
        __init__(dvbs2_modulator_bc_sptr self, dvbs2_modulator_bc p) -> dvbs2_modulator_bc_sptr
        """
        _dtv_swig1.dvbs2_modulator_bc_sptr_swiginit(self, _dtv_swig1.new_dvbs2_modulator_bc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbs2_modulator_bc *":
        r"""__deref__(dvbs2_modulator_bc_sptr self) -> dvbs2_modulator_bc"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbs2_modulator_bc_sptr

    def make(self, framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", interpolation: "gr::dtv::dvbs2_interpolation_t") -> "gr::dtv::dvbs2_modulator_bc::sptr":
        r"""make(dvbs2_modulator_bc_sptr self, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbs2_interpolation_t interpolation) -> dvbs2_modulator_bc_sptr"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_make(self, framesize, rate, constellation, interpolation)

    def history(self) -> "unsigned int":
        r"""history(dvbs2_modulator_bc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbs2_modulator_bc_sptr self, int which, int delay)
        declare_sample_delay(dvbs2_modulator_bc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbs2_modulator_bc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbs2_modulator_bc_sptr self, int multiple)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbs2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbs2_modulator_bc_sptr self) -> double"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbs2_modulator_bc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbs2_modulator_bc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbs2_modulator_bc_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbs2_modulator_bc_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbs2_modulator_bc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbs2_modulator_bc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbs2_modulator_bc_sptr self, std::string level)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbs2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbs2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbs2_modulator_bc_sptr self, int m)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbs2_modulator_bc_sptr self)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbs2_modulator_bc_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbs2_modulator_bc_sptr self, int m)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbs2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbs2_modulator_bc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbs2_modulator_bc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbs2_modulator_bc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbs2_modulator_bc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbs2_modulator_bc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbs2_modulator_bc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbs2_modulator_bc_sptr self, int which) -> float
        pc_input_buffers_full(dvbs2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbs2_modulator_bc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbs2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbs2_modulator_bc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbs2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbs2_modulator_bc_sptr self, int which) -> float
        pc_output_buffers_full(dvbs2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbs2_modulator_bc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbs2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbs2_modulator_bc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbs2_modulator_bc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbs2_modulator_bc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbs2_modulator_bc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbs2_modulator_bc_sptr self)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbs2_modulator_bc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbs2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbs2_modulator_bc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbs2_modulator_bc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbs2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbs2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbs2_modulator_bc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbs2_modulator_bc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbs2_modulator_bc_sptr self) -> long"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbs2_modulator_bc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbs2_modulator_bc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbs2_modulator_bc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbs2_modulator_bc_sptr self, std::string name)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbs2_modulator_bc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbs2_modulator_bc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbs2_modulator_bc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbs2_modulator_bc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_modulator_bc_sptr_message_subscribers(self, which_port)

# Register dvbs2_modulator_bc_sptr in _dtv_swig1:
_dtv_swig1.dvbs2_modulator_bc_sptr_swigregister(dvbs2_modulator_bc_sptr)


dvbs2_modulator_bc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbs2_modulator_bc = dvbs2_modulator_bc.make;

class dvbs2_physical_cc_sptr(object):
    r"""Proxy of C++ boost::shared_ptr< gr::dtv::dvbs2_physical_cc > class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        __init__(dvbs2_physical_cc_sptr self) -> dvbs2_physical_cc_sptr
        __init__(dvbs2_physical_cc_sptr self, dvbs2_physical_cc p) -> dvbs2_physical_cc_sptr
        """
        _dtv_swig1.dvbs2_physical_cc_sptr_swiginit(self, _dtv_swig1.new_dvbs2_physical_cc_sptr(*args))

    def __deref__(self) -> "gr::dtv::dvbs2_physical_cc *":
        r"""__deref__(dvbs2_physical_cc_sptr self) -> dvbs2_physical_cc"""
        return _dtv_swig1.dvbs2_physical_cc_sptr___deref__(self)
    __swig_destroy__ = _dtv_swig1.delete_dvbs2_physical_cc_sptr

    def make(self, framesize: "gr::dtv::dvb_framesize_t", rate: "gr::dtv::dvb_code_rate_t", constellation: "gr::dtv::dvb_constellation_t", pilots: "gr::dtv::dvbs2_pilots_t", goldcode: "int") -> "gr::dtv::dvbs2_physical_cc::sptr":
        r"""make(dvbs2_physical_cc_sptr self, gr::dtv::dvb_framesize_t framesize, gr::dtv::dvb_code_rate_t rate, gr::dtv::dvb_constellation_t constellation, gr::dtv::dvbs2_pilots_t pilots, int goldcode) -> dvbs2_physical_cc_sptr"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_make(self, framesize, rate, constellation, pilots, goldcode)

    def history(self) -> "unsigned int":
        r"""history(dvbs2_physical_cc_sptr self) -> unsigned int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_history(self)

    def declare_sample_delay(self, *args) -> "void":
        r"""
        declare_sample_delay(dvbs2_physical_cc_sptr self, int which, int delay)
        declare_sample_delay(dvbs2_physical_cc_sptr self, unsigned int delay)
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_declare_sample_delay(self, *args)

    def sample_delay(self, which: "int") -> "unsigned int":
        r"""sample_delay(dvbs2_physical_cc_sptr self, int which) -> unsigned int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_sample_delay(self, which)

    def set_output_multiple(self, multiple: "int") -> "void":
        r"""set_output_multiple(dvbs2_physical_cc_sptr self, int multiple)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_output_multiple(self, multiple)

    def output_multiple(self) -> "int":
        r"""output_multiple(dvbs2_physical_cc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_output_multiple(self)

    def relative_rate(self) -> "double":
        r"""relative_rate(dvbs2_physical_cc_sptr self) -> double"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_relative_rate(self)

    def relative_rate_i(self) -> "uint64_t":
        r"""relative_rate_i(dvbs2_physical_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_relative_rate_i(self)

    def relative_rate_d(self) -> "uint64_t":
        r"""relative_rate_d(dvbs2_physical_cc_sptr self) -> uint64_t"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_relative_rate_d(self)

    def start(self) -> "bool":
        r"""start(dvbs2_physical_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_start(self)

    def stop(self) -> "bool":
        r"""stop(dvbs2_physical_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_stop(self)

    def nitems_read(self, which_input: "unsigned int") -> "uint64_t":
        r"""nitems_read(dvbs2_physical_cc_sptr self, unsigned int which_input) -> uint64_t"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_nitems_read(self, which_input)

    def nitems_written(self, which_output: "unsigned int") -> "uint64_t":
        r"""nitems_written(dvbs2_physical_cc_sptr self, unsigned int which_output) -> uint64_t"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_nitems_written(self, which_output)

    def set_log_level(self, level: "std::string") -> "void":
        r"""set_log_level(dvbs2_physical_cc_sptr self, std::string level)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_log_level(self, level)

    def log_level(self) -> "std::string":
        r"""log_level(dvbs2_physical_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_log_level(self)

    def max_noutput_items(self) -> "int":
        r"""max_noutput_items(dvbs2_physical_cc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_max_noutput_items(self)

    def set_max_noutput_items(self, m: "int") -> "void":
        r"""set_max_noutput_items(dvbs2_physical_cc_sptr self, int m)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_max_noutput_items(self, m)

    def unset_max_noutput_items(self) -> "void":
        r"""unset_max_noutput_items(dvbs2_physical_cc_sptr self)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_unset_max_noutput_items(self)

    def is_set_max_noutput_items(self) -> "bool":
        r"""is_set_max_noutput_items(dvbs2_physical_cc_sptr self) -> bool"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_is_set_max_noutput_items(self)

    def set_min_noutput_items(self, m: "int") -> "void":
        r"""set_min_noutput_items(dvbs2_physical_cc_sptr self, int m)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_min_noutput_items(self, m)

    def min_noutput_items(self) -> "int":
        r"""min_noutput_items(dvbs2_physical_cc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_min_noutput_items(self)

    def max_output_buffer(self, i: "int") -> "long":
        r"""max_output_buffer(dvbs2_physical_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_max_output_buffer(self, i)

    def set_max_output_buffer(self, *args) -> "void":
        r"""
        set_max_output_buffer(dvbs2_physical_cc_sptr self, long max_output_buffer)
        set_max_output_buffer(dvbs2_physical_cc_sptr self, int port, long max_output_buffer)
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_max_output_buffer(self, *args)

    def min_output_buffer(self, i: "int") -> "long":
        r"""min_output_buffer(dvbs2_physical_cc_sptr self, int i) -> long"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_min_output_buffer(self, i)

    def set_min_output_buffer(self, *args) -> "void":
        r"""
        set_min_output_buffer(dvbs2_physical_cc_sptr self, long min_output_buffer)
        set_min_output_buffer(dvbs2_physical_cc_sptr self, int port, long min_output_buffer)
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_min_output_buffer(self, *args)

    def pc_noutput_items(self) -> "float":
        r"""pc_noutput_items(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_noutput_items(self)

    def pc_noutput_items_avg(self) -> "float":
        r"""pc_noutput_items_avg(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_noutput_items_avg(self)

    def pc_noutput_items_var(self) -> "float":
        r"""pc_noutput_items_var(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_noutput_items_var(self)

    def pc_nproduced(self) -> "float":
        r"""pc_nproduced(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_nproduced(self)

    def pc_nproduced_avg(self) -> "float":
        r"""pc_nproduced_avg(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_nproduced_avg(self)

    def pc_nproduced_var(self) -> "float":
        r"""pc_nproduced_var(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_nproduced_var(self)

    def pc_input_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full(dvbs2_physical_cc_sptr self, int which) -> float
        pc_input_buffers_full(dvbs2_physical_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_input_buffers_full(self, *args)

    def pc_input_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_avg(dvbs2_physical_cc_sptr self, int which) -> float
        pc_input_buffers_full_avg(dvbs2_physical_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_input_buffers_full_avg(self, *args)

    def pc_input_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_input_buffers_full_var(dvbs2_physical_cc_sptr self, int which) -> float
        pc_input_buffers_full_var(dvbs2_physical_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_input_buffers_full_var(self, *args)

    def pc_output_buffers_full(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full(dvbs2_physical_cc_sptr self, int which) -> float
        pc_output_buffers_full(dvbs2_physical_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_output_buffers_full(self, *args)

    def pc_output_buffers_full_avg(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_avg(dvbs2_physical_cc_sptr self, int which) -> float
        pc_output_buffers_full_avg(dvbs2_physical_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_output_buffers_full_avg(self, *args)

    def pc_output_buffers_full_var(self, *args) -> "std::vector< float,std::allocator< float > >":
        r"""
        pc_output_buffers_full_var(dvbs2_physical_cc_sptr self, int which) -> float
        pc_output_buffers_full_var(dvbs2_physical_cc_sptr self) -> pmt_vector_float
        """
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_output_buffers_full_var(self, *args)

    def pc_work_time(self) -> "float":
        r"""pc_work_time(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_work_time(self)

    def pc_work_time_avg(self) -> "float":
        r"""pc_work_time_avg(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_work_time_avg(self)

    def pc_work_time_var(self) -> "float":
        r"""pc_work_time_var(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_work_time_var(self)

    def pc_work_time_total(self) -> "float":
        r"""pc_work_time_total(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_work_time_total(self)

    def pc_throughput_avg(self) -> "float":
        r"""pc_throughput_avg(dvbs2_physical_cc_sptr self) -> float"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_pc_throughput_avg(self)

    def set_processor_affinity(self, mask: "std::vector< int,std::allocator< int > > const &") -> "void":
        r"""set_processor_affinity(dvbs2_physical_cc_sptr self, std::vector< int,std::allocator< int > > const & mask)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_processor_affinity(self, mask)

    def unset_processor_affinity(self) -> "void":
        r"""unset_processor_affinity(dvbs2_physical_cc_sptr self)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_unset_processor_affinity(self)

    def processor_affinity(self) -> "std::vector< int,std::allocator< int > >":
        r"""processor_affinity(dvbs2_physical_cc_sptr self) -> std::vector< int,std::allocator< int > >"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_processor_affinity(self)

    def active_thread_priority(self) -> "int":
        r"""active_thread_priority(dvbs2_physical_cc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_active_thread_priority(self)

    def thread_priority(self) -> "int":
        r"""thread_priority(dvbs2_physical_cc_sptr self) -> int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_thread_priority(self)

    def set_thread_priority(self, priority: "int") -> "int":
        r"""set_thread_priority(dvbs2_physical_cc_sptr self, int priority) -> int"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_thread_priority(self, priority)

    def name(self) -> "std::string":
        r"""name(dvbs2_physical_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_name(self)

    def symbol_name(self) -> "std::string":
        r"""symbol_name(dvbs2_physical_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_symbol_name(self)

    def input_signature(self) -> "gr::io_signature::sptr":
        r"""input_signature(dvbs2_physical_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_input_signature(self)

    def output_signature(self) -> "gr::io_signature::sptr":
        r"""output_signature(dvbs2_physical_cc_sptr self) -> io_signature_sptr"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_output_signature(self)

    def unique_id(self) -> "long":
        r"""unique_id(dvbs2_physical_cc_sptr self) -> long"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_unique_id(self)

    def to_basic_block(self) -> "gr::basic_block_sptr":
        r"""to_basic_block(dvbs2_physical_cc_sptr self) -> basic_block_sptr"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_to_basic_block(self)

    def check_topology(self, ninputs: "int", noutputs: "int") -> "bool":
        r"""check_topology(dvbs2_physical_cc_sptr self, int ninputs, int noutputs) -> bool"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_check_topology(self, ninputs, noutputs)

    def alias(self) -> "std::string":
        r"""alias(dvbs2_physical_cc_sptr self) -> std::string"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_alias(self)

    def set_block_alias(self, name: "std::string") -> "void":
        r"""set_block_alias(dvbs2_physical_cc_sptr self, std::string name)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_set_block_alias(self, name)

    def _post(self, which_port: "swig_pmt_ptr", msg: "swig_pmt_ptr") -> "void":
        r"""_post(dvbs2_physical_cc_sptr self, swig_pmt_ptr which_port, swig_pmt_ptr msg)"""
        return _dtv_swig1.dvbs2_physical_cc_sptr__post(self, which_port, msg)

    def message_ports_in(self) -> "pmt::pmt_t":
        r"""message_ports_in(dvbs2_physical_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_message_ports_in(self)

    def message_ports_out(self) -> "pmt::pmt_t":
        r"""message_ports_out(dvbs2_physical_cc_sptr self) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_message_ports_out(self)

    def message_subscribers(self, which_port: "swig_pmt_ptr") -> "pmt::pmt_t":
        r"""message_subscribers(dvbs2_physical_cc_sptr self, swig_pmt_ptr which_port) -> swig_pmt_ptr"""
        return _dtv_swig1.dvbs2_physical_cc_sptr_message_subscribers(self, which_port)

# Register dvbs2_physical_cc_sptr in _dtv_swig1:
_dtv_swig1.dvbs2_physical_cc_sptr_swigregister(dvbs2_physical_cc_sptr)


dvbs2_physical_cc_sptr.__repr__ = lambda self: "<gr_block %s (%d)>" % (self.name(), self.unique_id())
dvbs2_physical_cc = dvbs2_physical_cc.make;



